//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Nicolas Froment, Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version:  "1.0"
      description: "This plugin parses every lyric for a hyphen or underscore, converting them into syllabic or melismatic lyrics, respectively."
      menuPath: "Plugins.Notes.Fix Hyphens and Melismas"

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      function applyToNotesInSelection() {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score
                        var prevLyric
                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {      
                              if (cursor.element && cursor.element.type === Element.CHORD) { // go through every chord
                                    var lyrics = cursor.element.lyrics;
                                    var removed = 0
                                    for (var i = 0; i < lyrics.length; i++) { // if there are lyrics
                                          var lyric = lyrics[i];
                                          if (lyric.text === "-" || lyric.text === "_") { // single hyphen or underscore
                                                cursor.element.remove(lyric)
                                                removed = 1
                                          } else if (lyric.text.indexOf("-") !== -1) { // hyphen = start or continuation of multisyllabic word
                                                lyric.text = lyric.text.split("-")[0]
                                                if (prevLyric && (prevLyric.syllabic === Lyrics.MIDDLE || prevLyric.syllabic === Lyrics.BEGIN)) {
                                                      lyric.syllabic = Lyrics.MIDDLE
                                                } else {
                                                      lyric.syllabic = Lyrics.BEGIN
                                                }
                                                prevLyric = lyric
                                          } else if (lyric.text.indexOf("_") !== -1) { // underscore = start of melismatic word or syllable
                                                lyric.text = lyric.text.split("_")[0]
                                                if (prevLyric && (prevLyric.syllabic === Lyrics.MIDDLE || prevLyric.syllabic === Lyrics.BEGIN)) {
                                                      lyric.syllabic = Lyrics.END
                                                } else {
                                                      lyric.syllabic = Lyrics.SINGLE
                                                }
                                                prevLyric = lyric
                                          } else if (lyric.text !== "") { // single or end of multisyllablic word
                                                if (prevLyric && (prevLyric.syllabic === Lyrics.MIDDLE || prevLyric.syllabic === Lyrics.BEGIN)) {
                                                      lyric.syllabic = Lyrics.END
                                                } else {
                                                      lyric.syllabic = Lyrics.SINGLE
                                                }
                                                prevLyric = lyric
                                          }
                                    }
                                    if (!lyrics || lyrics.length === 0 || removed) { // no lyrics; should be either melisma or syllable
                                          removed = 0
                                          if(prevLyric) {
                                                if (prevLyric.syllabic === Lyrics.BEGIN || prevLyric.syllabic === Lyrics.MIDDLE) { // is part of multisyllabic word
                                                
                                                } else { // is melisma
                                                      prevLyric.ticks = cursor.element.parent.tick - prevLyric.parent.parent.tick
                                                }
                                          }
                                    }
                              }
                              if (cursor.element && cursor.element.type === Element.REST) { // assume syllables can't carry over rests
                                    prevLyric = 0
                              }
                        cursor.next();
                        }
                  }
            }
      }

      onRun: {
            if (typeof curScore === 'undefined')
                  Qt.quit();

            applyToNotesInSelection()

            Qt.quit();
         }
}
